package ru.dbotthepony.mc.otm.client.render

import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import com.mojang.serialization.codecs.RecordCodecBuilder
import net.minecraft.client.renderer.texture.SpriteContents
import net.minecraft.client.renderer.texture.atlas.SpriteResourceLoader
import net.minecraft.client.renderer.texture.atlas.SpriteSource
import net.minecraft.client.renderer.texture.atlas.SpriteSource.TEXTURE_ID_CONVERTER
import net.minecraft.client.renderer.texture.atlas.SpriteSourceType
import net.minecraft.client.renderer.texture.atlas.sources.LazyLoadedImage
import net.minecraft.client.resources.metadata.animation.FrameSize
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.packs.resources.ResourceManager
import net.minecraft.server.packs.resources.ResourceMetadata
import net.minecraft.world.item.DyeColor
import net.neoforged.neoforge.client.event.RegisterSpriteSourceTypesEvent
import ru.dbotthepony.kommons.math.RGBAColor
import ru.dbotthepony.mc.otm.OverdriveThatMatters.loc

class ColorizedSpriteSource(val textures: Map<ResourceLocation, ResourceLocation>) : SpriteSource {
	override fun run(
		manager: ResourceManager,
		output: SpriteSource.Output
	) {
		for (entry in textures) {
			val baseRes = manager.getResource(TEXTURE_ID_CONVERTER.idToFile(entry.key)).get()
			val maskRes = manager.getResource(TEXTURE_ID_CONVERTER.idToFile(entry.value)).get()

			DyeColor.entries.forEach {
				val outputLocation = entry.key.withSuffix("/" + it.serializedName)

				val baseImage = LazyLoadedImage(entry.key, baseRes, 1)
				val maskImage = LazyLoadedImage(entry.value, maskRes, 1)

				output.add(outputLocation, Supplier(baseImage, maskImage, it, outputLocation))
			}
		}
	}

	override fun type(): SpriteSourceType {
		return SPRITE_SOURCE!!
	}

	class Supplier(val baseImage: LazyLoadedImage, val maskImage: LazyLoadedImage, val color: DyeColor, val outputLocation: ResourceLocation) : SpriteSource.SpriteSupplier {
		override fun apply(t: SpriteResourceLoader?): SpriteContents? {
			val base = baseImage.get()
			val mask = maskImage.get()

			if (base.width != mask.width || base.height != mask.height) {
				throw IllegalArgumentException("Base image doesn't match mask in texture size")
			}

			val diffCol = RGBAColor.argb(color.textureDiffuseColor)
			mask.applyToAllPixels {
				val a = (it and -0x1000000 ushr 24) / 255f
				val b = (it and 0xFF0000 ushr 16) / 255f
				val g = (it and 0xFF00 ushr 8) / 255f
				val r = (it and 0xFF) / 255f

				RGBAColor(r * diffCol.red, g * diffCol.green, b * diffCol.blue, a).toRGBA()
			}

			for (i in 0 until base.width) {
				for (j in 0 until base.height) {
					base.blendPixel(i, j, mask.getPixelRGBA(i, j))
				}
			}

			maskImage.release()

			return SpriteContents(outputLocation, FrameSize(base.width, base.height), base, ResourceMetadata.EMPTY)
		}

	}

	companion object {
		val LOCATION = loc("colorized")

		private val CODEC: MapCodec<ColorizedSpriteSource> = RecordCodecBuilder.mapCodec {
			it.group(
				Codec.unboundedMap(ResourceLocation.CODEC, ResourceLocation.CODEC).fieldOf("textures").forGetter { it.textures }
			).apply(it, ::ColorizedSpriteSource)
		}

		var SPRITE_SOURCE: SpriteSourceType? = null

		fun register(event: RegisterSpriteSourceTypesEvent) {
			SPRITE_SOURCE = event.register(LOCATION, CODEC)
		}
	}
}
